package com.coresun.commons.struts.validator;

import java.io.Serializable;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.commons.validator.Field;
import org.apache.commons.validator.Validator;
import org.apache.commons.validator.ValidatorAction;
import org.apache.struts.action.ActionMessages;
import org.apache.struts.upload.FormFile;
import org.apache.struts.validator.Resources;

import com.coresun.commons.validator.FormFileValidator;

/**
 * <p>This class contains the validation methods for the file field.</p>
 * 
 * @author Utku Utkan
 */
public class FileFieldChecks implements Serializable {
	
	private static final long serialVersionUID = 8472582664903583879L;
	
	private static final Log log = LogFactory.getLog(FileFieldChecks.class);
	
	/**
	 * Checks if the file size is more than 0.
	 * 
	 * @param bean The bean validation is being performed on.
	 * @param va The <code>ValidatorAction</code> that is currently being performed.
	 * @param field The <code>Field</code> object associated with the current field being validated.
	 * @param errors The <code>ActionMessages</code> object to add errors to if any validation errors occur.
	 * @param validator The <code>Validator</code> instance, used to access other field values.
	 * @param request Current request object.
	 * @return
	 */
	public static boolean validateRequiredFile(Object bean, ValidatorAction va, Field field, ActionMessages errors, Validator validator, HttpServletRequest request) {
		FormFile formFile = null;
		
		try {
			formFile = (FormFile) PropertyUtils.getProperty(bean, field.getProperty());
		} catch (Exception e) {
			log.error("Bean provided does not contain a property named '" + field.getProperty() + "' of type " + FormFile.class.getName(), e);
			throw new IllegalArgumentException("Bean provided does not contain a property named '" + field.getProperty() + "' of type " + FormFile.class.getName(), e);
		}
		
		if (FormFileValidator.validateRequiredFile(formFile)) {
			return true;
		} else {
			errors.add(field.getKey(), Resources.getActionMessage(validator, request, va, field));
			
			return false;
		}
	}
	
	/**
	 * Checks if the file size is more than the minimum value.
	 * 
	 * @param bean The bean validation is being performed on.
	 * @param va The <code>ValidatorAction</code> that is currently being performed.
	 * @param field The <code>Field</code> object associated with the current field being validated.
	 * @param errors The <code>ActionMessages</code> object to add errors to if any validation errors occur.
	 * @param validator The <code>Validator</code> instance, used to access other field values.
	 * @param request Current request object.
	 * @return
	 */
	public static boolean validateMinFileSize(Object bean, ValidatorAction va, Field field, ActionMessages errors, Validator validator, HttpServletRequest request) {
		FormFile formFile = null;
		int min = 0;
		FormFileValidator.Unit unit = null;
		
		try {
			formFile = (FormFile) PropertyUtils.getProperty(bean, field.getProperty());
		} catch (Exception e) {
			log.error("Bean provided does not contain a property named '" + field.getProperty() + "' of type " + FormFile.class.getName(), e);
			throw new IllegalArgumentException("Bean provided does not contain a property named '" + field.getProperty() + "' of type " + FormFile.class.getName(), e);
		}
		
		try {
			min = Integer.parseInt(field.getVarValue("min"));
		} catch (NumberFormatException e) {
			log.error("Variable 'min' does not contain a valid int value.", e);
			throw new IllegalArgumentException("Variable 'min' does not contain a valid int value.", e);
		}
		
		String unitString = field.getVarValue("unit");
		
		if (unitString == null) {
			log.error("Variable 'unit' does not contain a valid file size unit.");
			throw new IllegalArgumentException("Variable 'unit' does not contain a valid file size unit.");
		} else if (unitString.equals("KB")) {
			unit = FormFileValidator.Unit.KB;
		} else if (unitString.equals("MB")) {
			unit = FormFileValidator.Unit.MB;
		} else if (unitString.equals("GB")) {
			unit = FormFileValidator.Unit.GB;
		} else {
			log.error("Variable 'unit' does not contain a valid file size unit.");
			throw new IllegalArgumentException("Variable 'unit' does not contain a valid file size unit.");
		}
		
		if (FormFileValidator.validateMaxFileSize(formFile, min, unit)) {
			return true;
		} else {
			errors.add(field.getKey(), Resources.getActionMessage(validator, request, va, field));
			
			return false;
		}
	}
	
	/**
	 * Checks if the file size is less than the maximum value.
	 * 
	 * @param bean The bean validation is being performed on.
	 * @param va The <code>ValidatorAction</code> that is currently being performed.
	 * @param field The <code>Field</code> object associated with the current field being validated.
	 * @param errors The <code>ActionMessages</code> object to add errors to if any validation errors occur.
	 * @param validator The <code>Validator</code> instance, used to access other field values.
	 * @param request Current request object.
	 * @return
	 */
	public static boolean validateMaxFileSize(Object bean, ValidatorAction va, Field field, ActionMessages errors, Validator validator, HttpServletRequest request) {
		FormFile formFile = null;
		int max = 0;
		FormFileValidator.Unit unit = null;
		
		try {
			formFile = (FormFile) PropertyUtils.getProperty(bean, field.getProperty());
		} catch (Exception e) {
			log.error("Bean provided does not contain a property named '" + field.getProperty() + "' of type " + FormFile.class.getName(), e);
			throw new IllegalArgumentException("Bean provided does not contain a property named '" + field.getProperty() + "' of type " + FormFile.class.getName(), e);
		}
		
		try {
			max = Integer.parseInt(field.getVarValue("max"));
		} catch (NumberFormatException e) {
			log.error("Variable 'max' does not contain a valid int value.", e);
			throw new IllegalArgumentException("Variable 'max' does not contain a valid int value.", e);
		}
		
		String unitString = field.getVarValue("unit");
		
		if (unitString == null) {
			log.error("Variable 'unit' does not contain a valid file size unit.");
			throw new IllegalArgumentException("Variable 'unit' does not contain a valid file size unit.");
		} else if (unitString.equals("KB")) {
			unit = FormFileValidator.Unit.KB;
		} else if (unitString.equals("MB")) {
			unit = FormFileValidator.Unit.MB;
		} else if (unitString.equals("GB")) {
			unit = FormFileValidator.Unit.GB;
		} else {
			log.error("Variable 'unit' does not contain a valid file size unit.");
			throw new IllegalArgumentException("Variable 'unit' does not contain a valid file size unit.");
		}
		
		if (FormFileValidator.validateMaxFileSize(formFile, max, unit)) {
			return true;
		} else {
			errors.add(field.getKey(), Resources.getActionMessage(validator, request, va, field));
			
			return false;
		}
	}
	
	/**
	 * Checks if the content type is one of the valid content types.
	 * 
	 * @param bean The bean validation is being performed on.
	 * @param va The <code>ValidatorAction</code> that is currently being performed.
	 * @param field The <code>Field</code> object associated with the current field being validated.
	 * @param errors The <code>ActionMessages</code> object to add errors to if any validation errors occur.
	 * @param validator The <code>Validator</code> instance, used to access other field values.
	 * @param request Current request object.
	 * @return
	 */
	public static boolean validateFileContent(Object bean, ValidatorAction va, Field field, ActionMessages errors, Validator validator, HttpServletRequest request) {
		FormFile formFile = null;
		
		try {
			formFile = (FormFile) PropertyUtils.getProperty(bean, field.getProperty());
		} catch (Exception e) {
			log.error("Bean provided does not contain a property named '" + field.getProperty() + "' of type " + FormFile.class.getName(), e);
			throw new IllegalArgumentException("Bean provided does not contain a property named '" + field.getProperty() + "' of type " + FormFile.class.getName(), e);
		}
		
		String[] contentTypes = field.getVarValue("contentTypes").split(",");
		
		if (FormFileValidator.validateFileContent(formFile, contentTypes)) {
			return true;
		} else {
			errors.add(field.getKey(), Resources.getActionMessage(validator, request, va, field));
			
			return false;
		}
	}
}
