package com.coresun.commons.validator;

import java.io.Serializable;

import org.apache.struts.upload.FormFile;

/**
 * <p>Perform form file validations.</p>
 * 
 * @author Utku Utkan
 */
public class FormFileValidator implements Serializable {
	
	private static final long serialVersionUID = 8472582664903583879L;
	
	public enum Unit { KB, MB, GB };
	
	/**
	 * Converts the file size in the given unit to Bytes.
	 * 
	 * @param size Size of the file in a given unit.
	 * @param unit Unit of the file size: KB, MB or GB.
	 * @return Size of the file in Bytes.
	 */
	private static int convertToBytes(int size, Unit unit) {
		switch (unit) {
			case KB:
				return size * 1024;
			case MB:
				return size * 1048576;
			case GB:
				return size * 1073741824;
			default:
				return size;
		}
	}
	
	/**
	 * Checks if the file size is more than 0.
	 * 
	 * @param formFile The form file validation is being performed on.
	 * @return <code>true</code> if the size of the file is more than 0.
	 */
	public static boolean validateRequiredFile(FormFile formFile) {
		if (formFile.getFileSize() > 0) {
			return true;
		} else {
			return false;
		}
	}
	
	/**
	 * Checks if the file size is more than the min.
	 * 
	 * @param formFile The form file validation is being performed on.
	 * @param min The minimum file size.
	 * @param unit Unit of the file size: KB, MB or GB.
	 * @return <code>true</code> if the size of the file is more than the specified minimum.
	 */
	public static boolean validateMinFileSize(FormFile formFile, int min, Unit unit) {
		if (formFile.getFileSize() < convertToBytes(min, unit)) {
			return false;
		} else {
			return true;
		}
	}
	
	/**
	 * Checks if the file size is less than the max.
	 * 
	 * @param formFile The form file validation is being performed on.
	 * @param max The maximum file size.
	 * @param unit Unit of the file size: KB, MB or GB.
	 * @return <code>true</code> if the size of the file is less than the specified maximum.
	 */
	public static boolean validateMaxFileSize(FormFile formFile, int max, Unit unit) {
		if (formFile.getFileSize() > convertToBytes(max, unit)) {
			return false;
		} else {
			return true;
		}
	}
	
	/**
	 * Checks if the content type is one of the valid content types.
	 * 
	 * @param formFile The form file validation is being performed on.
	 * @param contentTypes Array of the valid content types.
	 * @return <code>true</code> if the content type of the file is a valid one.
	 */
	public static boolean validateFileContent(FormFile formFile, String[] contentTypes) {
		if (formFile.getFileSize() == 0) {
			return true;
		}
		
		String contentType = formFile.getContentType();
		
		for (int i = 0; i < contentTypes.length; i++) {
			if (contentType.equals(contentTypes[i])) {
				return true;
			}
		}
		
		return false;
	}
}
